
/* Chrysalide - Outil d'analyse de fichiers binaires
 * pipe.h - prototypes pour la gestion de combinaisons de transformateurs
 *
 * Copyright (C) 2023 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _ANALYSIS_SCAN_MODIFIERS_PIPE_H
#define _ANALYSIS_SCAN_MODIFIERS_PIPE_H


#include <glib-object.h>
#include <stdbool.h>


#include "../modifier.h"



#define G_TYPE_SCAN_MODIFIER_PIPE            g_scan_modifier_pipe_get_type()
#define G_SCAN_MODIFIER_PIPE(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), G_TYPE_SCAN_MODIFIER_PIPE, GScanModifierPipe))
#define G_IS_SCAN_MODIFIER_PIPE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), G_TYPE_SCAN_MODIFIER_PIPE))
#define G_SCAN_MODIFIER_PIPE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass), G_TYPE_SCAN_MODIFIER_PIPE, GScanModifierPipeClass))
#define G_IS_SCAN_MODIFIER_PIPE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), G_TYPE_SCAN_MODIFIER_PIPE))
#define G_SCAN_MODIFIER_PIPE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), G_TYPE_SCAN_MODIFIER_PIPE, GScanModifierPipeClass))


/* Enchainement combinatoire de transformations d'octets (instance) */
typedef struct _GScanModifierPipe GScanModifierPipe;

/* Enchainement combinatoire de transformations d'octets (classe) */
typedef struct _GScanModifierPipeClass GScanModifierPipeClass;


/* Indique le type défini pour une série de transformations d'octets. */
GType g_scan_modifier_pipe_get_type(void);

/* Construit une pipee de modificateurs d'octets. */
GScanTokenModifier *g_scan_modifier_pipe_new(void);

/* Intègre un nouveau transformateur à enchaîner. */
void g_scan_modifier_pipe_add(GScanModifierPipe *, GScanTokenModifier *);

/* Indique le nombre de transformateurs intégrés dans la série. */
size_t g_scan_modifier_pipe_count(const GScanModifierPipe *);

/* Fournit un transformateur donné de la pipee. */
GScanTokenModifier *g_scan_modifier_pipe_get(const GScanModifierPipe *, size_t);



#endif  /* _ANALYSIS_SCAN_MODIFIERS_PIPE_H */
