
/* Chrysalide - Outil d'analyse de fichiers binaires
 * node-int.h - prototypes internes pour la décomposition d'un motif de recherche en atomes assemblés
 *
 * Copyright (C) 2023 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#ifndef _ANALYSIS_SCAN_PATTERNS_TOKENS_NODE_INT_H
#define _ANALYSIS_SCAN_PATTERNS_TOKENS_NODE_INT_H


#include "node.h"



/* Communique l'intérêt d'un noeud au sein d'une analyse. */
typedef float (* compute_scan_token_node_weight_fc) (const GScanTokenNode *);

/* Prend acte d'une nouvelle propriété pour le noeud. */
typedef void (* apply_scan_token_node_flags_fc) (GScanTokenNode *, ScanTokenNodeFlags);

/* Noeuds clefs de l'arborescence mise en place */
typedef struct _scan_tree_points_t
{
    GScanTokenNode *first_plain;            /* Premier noeud textuel       */
    GScanTokenNode *best_masked;            /* Noeud masqué le plus long   */

} scan_tree_points_t;

/* Parcourt une arborescence de noeuds et y relève des éléments. */
typedef void (* visit_scan_token_node_fc) (GScanTokenNode *, scan_tree_points_t *);

/* Inscrit la définition d'un motif dans un moteur de recherche. */
typedef bool (* enroll_scan_token_node_fc) (GScanTokenNode *, GEngineBackend *, size_t, size_t *);

/* Récupère un identifiant final pour un atome d'octets. */
typedef bool (* build_scan_token_node_id_fc) (GScanTokenNode *, GEngineBackend *);

typedef enum _TokenNodeCheckFlags
{
    TNCF_UPDATE_IN_PLACE = (1 << 0),        /* Actualisation de l'entrée   */
    TNCF_CREATE_NEW      = (1 << 1),        /* Allocation de positions     */
    TNCF_KEEP_DISCARDED  = (1 << 2),        /* Inversion des résultats     */

} TokenNodeCheckFlags;

/* Transforme les correspondances locales en trouvailles. */
typedef void (* check_scan_token_node_fc) (const GScanTokenNode *, scan_node_check_params_t *, TokenNodeCheckFlags, bool *);


/* Décomposition d'un motif de recherche en atomes (instance) */
struct _GScanTokenNode
{
    GObject parent;                         /* A laisser en premier        */

    ScanTokenNodeFlags flags;               /* Propriétés particulières    */

};

/* Décomposition d'un motif de recherche en atomes (classe) */
struct _GScanTokenNodeClass
{
    GObjectClass parent;                    /* A laisser en premier        */

    compute_scan_token_node_weight_fc compute_weight; /* Evaluation        */
    visit_scan_token_node_fc visit;         /* Phase de répérage initial   */
    apply_scan_token_node_flags_fc apply;   /* Prise en compte de fanions  */

    enroll_scan_token_node_fc enroll;       /* Inscription d'un motif      */
    build_scan_token_node_id_fc build_id;   /* Récupération d'identifiant  */

    check_scan_token_node_fc check_forward; /* Conversion en trouvailles   */
    check_scan_token_node_fc check_backward;/* Conversion en trouvailles   */

};


/* Parcourt une arborescence de noeuds et y relève des éléments. */
void g_scan_token_node_visit(GScanTokenNode *, scan_tree_points_t *);

/* Inscrit la définition d'un motif dans un moteur de recherche. */
bool _g_scan_token_node_enroll(GScanTokenNode *, GEngineBackend *, size_t, size_t *);

/* Transforme les correspondances locales en trouvailles. */
void _g_scan_token_node_check_forward(const GScanTokenNode *, scan_node_check_params_t *, TokenNodeCheckFlags, bool *);

/* Transforme les correspondances locales en trouvailles. */
void _g_scan_token_node_check_backward(const GScanTokenNode *, scan_node_check_params_t *, TokenNodeCheckFlags, bool *);



#endif  /* _ANALYSIS_SCAN_PATTERNS_TOKENS_NODE_INT_H */
